@@ -380,7 +380,7 @@ KateBuildView::KateBuildView(KateBuildPlugin *plugin, KTextEditor::MainWindow *m
     m_buildUi.textBrowser->setReadOnly(true);
     m_buildUi.textBrowser->setOpenLinks(false);
     connect(m_buildUi.textBrowser, &QTextBrowser::anchorClicked, this, [this](const QUrl &url) {
-        static QRegularExpression fileRegExp(QStringLiteral("(.*):(\\d+):(\\d+)"));
+        static const QRegularExpression fileRegExp(QStringLiteral("(.*):(\\d+):(\\d+)"));
         const auto match = fileRegExp.match(url.toString(QUrl::None));
         if (!match.hasMatch() || !m_win) {
             return;
@@ -561,8 +561,6 @@ void KateBuildView::readSessionConfig(const KConfigGroup &cg)
     // Add project targets, if any
     addProjectTargets();
 
-    m_targetsUi->targetsView->expandAll();
-
     // pre-select the last active target or the first target of the first set
     int prevTargetSetRow = cg.readEntry(QStringLiteral("Active Target Index"), 0);
     int prevCmdRow = cg.readEntry(QStringLiteral("Active Target Command"), 0);
@@ -1253,7 +1251,7 @@ bool KateBuildView::isCommandLineAllowed(const QStringList &cmdline)
     }
 
     // ask user if the start should be allowed
-    QPointer<QMessageBox> msgBox(new QMessageBox(QApplication::activeWindow()));
+    QPointer<QMessageBox> msgBox(new QMessageBox(m_win->window()));
     msgBox->setWindowTitle(i18n("Build plugin wants to execute program"));
     msgBox->setTextFormat(Qt::RichText);
     msgBox->setText(
@@ -1599,15 +1597,15 @@ KateBuildView::OutputLine KateBuildView::processOutputLine(QString line)
     }
 
     Category category = Category::Normal;
-    static QRegularExpression errorRegExp(QStringLiteral("error:"), QRegularExpression::CaseInsensitiveOption);
-    static QRegularExpression errorRegExpTr(QStringLiteral("%1:").arg(i18nc("The same word as 'gcc' uses for an error.", "error")),
-                                            QRegularExpression::CaseInsensitiveOption);
-    static QRegularExpression warningRegExp(QStringLiteral("warning:"), QRegularExpression::CaseInsensitiveOption);
-    static QRegularExpression warningRegExpTr(QStringLiteral("%1:").arg(i18nc("The same word as 'gcc' uses for a warning.", "warning")),
-                                              QRegularExpression::CaseInsensitiveOption);
-    static QRegularExpression infoRegExp(QStringLiteral("(info|note):"), QRegularExpression::CaseInsensitiveOption);
-    static QRegularExpression infoRegExpTr(QStringLiteral("(%1):").arg(i18nc("The same words as 'gcc' uses for note or info.", "note|info")),
-                                           QRegularExpression::CaseInsensitiveOption);
+    static const QRegularExpression errorRegExp(QStringLiteral("error:"), QRegularExpression::CaseInsensitiveOption);
+    static const QRegularExpression errorRegExpTr(QStringLiteral("%1:").arg(i18nc("The same word as 'gcc' uses for an error.", "error")),
+                                                  QRegularExpression::CaseInsensitiveOption);
+    static const QRegularExpression warningRegExp(QStringLiteral("warning:"), QRegularExpression::CaseInsensitiveOption);
+    static const QRegularExpression warningRegExpTr(QStringLiteral("%1:").arg(i18nc("The same word as 'gcc' uses for a warning.", "warning")),
+                                                    QRegularExpression::CaseInsensitiveOption);
+    static const QRegularExpression infoRegExp(QStringLiteral("(info|note):"), QRegularExpression::CaseInsensitiveOption);
+    static const QRegularExpression infoRegExpTr(QStringLiteral("(%1):").arg(i18nc("The same words as 'gcc' uses for note or info.", "note|info")),
+                                                 QRegularExpression::CaseInsensitiveOption);
     if (msg.contains(errorRegExp) || msg.contains(errorRegExpTr) || msg.contains(QLatin1String("undefined reference"))
         || msg.contains(i18nc("The same word as 'ld' uses to mark an ...", "undefined reference"))) {
         category = Category::Error;
