@@ -7,9 +7,15 @@
 
 #include "kateapp.h"
 
+#include "doc_or_widget.h"
+#include "kate_timings_debug.h"
 #include "katemainwindow.h"
 #include "kateviewmanager.h"
 
+#ifdef WITH_DBUS
+#include "kateappadaptor.h"
+#endif
+
 #include <KAboutData>
 #include <KConfigGui>
 #include <KCrash>
@@ -19,13 +25,19 @@
 #include <KMessageBox>
 #include <KNetworkMounts>
 #include <KSharedConfig>
+#include <KTextEditor/View>
 #include <KWindowSystem>
+#include <QLoggingCategory>
 
 #define HAVE_STYLE_MANAGER __has_include(<KStyleManager>)
 #if HAVE_STYLE_MANAGER
 #include <KStyleManager>
 #endif
 
+#ifdef WITH_KUSERFEEDBACK
+#include <KUserFeedback/Provider>
+#endif
+
 // signal handler for SIGINT & SIGTERM
 #ifdef Q_OS_UNIX
 #include <KSignalHandler>
@@ -96,6 +108,16 @@
 // remember if we were started inside a terminal
 static bool insideTerminal = false;
 
+#ifdef WITH_DBUS
+/**
+ * dbus interface, must survive longer than m_docManager
+ * e.g. the destroyed signal of the document might access this
+ * global static as we register one on the qApp instance
+ * Qt docs tell one shall not delete it, it will disappear with it's parent
+ */
+static KateAppAdaptor *adaptor = nullptr;
+#endif
+
 /**
  * singleton instance pointer
  */
@@ -137,7 +159,7 @@ void KateApp::initPreApplicationCreation(bool detach)
 #endif
 
 #if defined(Q_OS_WIN)
-    // Enable on windows to see output in console
+    // try to attach to console for terminal detection and output
     if (AttachConsole(ATTACH_PARENT_PROCESS)) {
         // we are inside a terminal
         insideTerminal = true;
@@ -148,6 +170,9 @@ void KateApp::initPreApplicationCreation(bool detach)
                 freopen("CON", "w", stdout);
             if (fileno(stderr) < 0)
                 freopen("CON", "w", stderr);
+        } else {
+            // detach again to not die on ctrl-c and Co.
+            FreeConsole();
         }
     }
 #endif
@@ -178,13 +203,20 @@ KateApp::KateApp(const QCommandLineParser &args, const ApplicationMode mode, con
     : m_args(args)
     , m_mode(mode)
     , m_wrapper(appSelf = this)
-#ifdef WITH_DBUS
-    , m_adaptor(this)
-#endif
     , m_docManager(this)
     , m_sessionManager(this, sessionsDir)
+#ifdef WITH_KUSERFEEDBACK
+    , m_userFeedbackProvider(new KUserFeedback::Provider(this))
+#endif
     , m_lastActivationChange(QDateTime::currentMSecsSinceEpoch())
 {
+#ifdef WITH_DBUS
+    // init adaptor if not already there, that can happen in tests
+    if (!adaptor) {
+        adaptor = new KateAppAdaptor;
+    }
+#endif
+
 #if HAVE_STYLE_MANAGER
     /**
      * trigger initialisation of proper application style
@@ -223,27 +255,27 @@ KateApp::KateApp(const QCommandLineParser &args, const ApplicationMode mode, con
      * defaults, inspired by plasma
      * important: choose between kate and kwrite mode here to submit the right product id
      */
-    m_userFeedbackProvider.setProductIdentifier(isKate() ? QStringLiteral("org.kde.kate") : QStringLiteral("org.kde.kwrite"));
-    m_userFeedbackProvider.setFeedbackServer(QUrl(QStringLiteral("https://telemetry.kde.org/")));
-    m_userFeedbackProvider.setSubmissionInterval(7);
-    m_userFeedbackProvider.setApplicationStartsUntilEncouragement(5);
-    m_userFeedbackProvider.setEncouragementDelay(30);
+    m_userFeedbackProvider->setProductIdentifier(isKate() ? QStringLiteral("org.kde.kate") : QStringLiteral("org.kde.kwrite"));
+    m_userFeedbackProvider->setFeedbackServer(QUrl(QStringLiteral("https://telemetry.kde.org/")));
+    m_userFeedbackProvider->setSubmissionInterval(7);
+    m_userFeedbackProvider->setApplicationStartsUntilEncouragement(5);
+    m_userFeedbackProvider->setEncouragementDelay(30);
 
     /**
      * add some feedback providers
      */
 
     // software version info
-    m_userFeedbackProvider.addDataSource(new KUserFeedback::ApplicationVersionSource);
-    m_userFeedbackProvider.addDataSource(new KUserFeedback::QtVersionSource);
+    m_userFeedbackProvider->addDataSource(new KUserFeedback::ApplicationVersionSource);
+    m_userFeedbackProvider->addDataSource(new KUserFeedback::QtVersionSource);
 
     // info about the machine
-    m_userFeedbackProvider.addDataSource(new KUserFeedback::PlatformInfoSource);
-    m_userFeedbackProvider.addDataSource(new KUserFeedback::ScreenInfoSource);
+    m_userFeedbackProvider->addDataSource(new KUserFeedback::PlatformInfoSource);
+    m_userFeedbackProvider->addDataSource(new KUserFeedback::ScreenInfoSource);
 
     // usage info
-    m_userFeedbackProvider.addDataSource(new KUserFeedback::StartCountSource);
-    m_userFeedbackProvider.addDataSource(new KUserFeedback::UsageTimeSource);
+    m_userFeedbackProvider->addDataSource(new KUserFeedback::StartCountSource);
+    m_userFeedbackProvider->addDataSource(new KUserFeedback::UsageTimeSource);
 #endif
 }
 
@@ -257,7 +289,7 @@ KateApp::~KateApp()
      * unregister from dbus before we get unusable...
      */
     if (QDBusConnection::sessionBus().interface()) {
-        m_adaptor.emitExiting();
+        adaptor->emitExiting();
         QDBusConnection::sessionBus().unregisterObject(QStringLiteral("/MainApplication"));
     }
 #endif
@@ -366,6 +398,13 @@ bool KateApp::init()
         }
     }
 
+    // Save session now, else if the user never opens a new doc or closes one session-autosave will never trigger and
+    // the session will only get saved if Kate is properly closed and not if it gets killed.
+    // Do it delayed, don't block the startup
+    QTimer::singleShot(100, this, [this] {
+        sessionManager()->saveActiveSession(true);
+    });
+
     return true;
 }
 
@@ -488,7 +527,6 @@ void KateApp::shutdownKate(KateMainWindow *win)
     }
 
     sessionManager()->saveActiveSession(true);
-    stashManager()->stashDocuments(sessionManager()->activeSession()->config(), documentManager()->documentList());
 
     /**
      * all main windows will be cleaned up
@@ -544,7 +582,7 @@ KTextEditor::Document *KateApp::openDocUrl(const QUrl &url, const QString &encod
     if (noDir) {
         KateDocumentInfo docInfo;
         docInfo.startCursor = c;
-        doc = mainWindow->viewManager()->openUrl(url, encoding, activateView, isTempFile, docInfo);
+        doc = mainWindow->viewManager()->openUrl(url, encoding, activateView, isTempFile, &docInfo);
     } else {
         KMessageBox::error(mainWindow, i18n("The file '%1' could not be opened: it is not a normal file, it is a folder.", url.url()));
     }
@@ -578,7 +616,7 @@ void KateApp::openDocUrlDocumentDestroyed(QObject *document)
 
 #ifdef WITH_DBUS
     // emit token signal to unblock remove blocking instances
-    m_adaptor.emitDocumentClosed(QString::number(reinterpret_cast<qptrdiff>(document)));
+    adaptor->emitDocumentClosed(QString::number(reinterpret_cast<qptrdiff>(document)));
 #endif
 }
 
@@ -681,7 +719,10 @@ KateMainWindow *KateApp::newMainWindow(KConfig *sconfig_, const QString &sgroup_
     KConfig *sconfig = sconfig_ ? sconfig_ : KSharedConfig::openConfig().data();
     QString sgroup = !sgroup_.isEmpty() ? sgroup_ : QStringLiteral("MainWindow0");
 
+    QElapsedTimer t;
+    t.start();
     KateMainWindow *mainWindow = new KateMainWindow(sconfig, sgroup, userTriggered);
+    qCDebug(LibKateTime, "Created KateMainWindow in %lld ms", t.elapsed());
     mainWindow->show();
 
     return mainWindow;
