@@ -113,10 +113,11 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     KAcceleratorManager::setNoAccel(m_quickOpen);
     hLayout->addWidget(m_quickOpen);
 
+    auto mwActionCollection = m_viewManager->mainWindow()->actionCollection();
     // forward tab bar quick open action to global quick open action
     QAction *bridge = new QAction(QIcon::fromTheme(QStringLiteral("quickopen")), i18nc("indicator for more documents", "+%1", 100), this);
     m_quickOpen->setDefaultAction(bridge);
-    QAction *quickOpen = m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_quick_open"));
+    QAction *quickOpen = mwActionCollection->action(QStringLiteral("view_quick_open"));
     Q_ASSERT(quickOpen);
     bridge->setToolTip(quickOpen->toolTip());
     bridge->setWhatsThis(i18n("Click here to switch to the Quick Open view."));
@@ -127,13 +128,14 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     m_split->setAutoRaise(true);
     m_split->setPopupMode(QToolButton::InstantPopup);
     m_split->setIcon(QIcon::fromTheme(QStringLiteral("view-split-left-right")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_split_vert")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_split_horiz")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_split_vert_move_doc")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_split_horiz_move_doc")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_close_current_space")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_close_others")));
-    m_split->addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_hide_others")));
+    m_split->setMenu(new QMenu(this));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_split_vert")));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_split_horiz")));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_split_vert_move_doc")));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_split_horiz_move_doc")));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_close_current_space")));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_close_others")));
+    m_split->menu()->addAction(mwActionCollection->action(QStringLiteral("view_hide_others")));
     m_split->setToolTip(i18n("Split View"));
     m_split->setWhatsThis(i18n("Control view space splitting"));
 
@@ -150,6 +152,15 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     m_split->installEventFilter(this);
     // END tab bar
 
+    // more explicit enabling of the view space, see bug 485210
+    // we need to setMenu above and watch for aboutToShow to get that right
+    connect(m_quickOpen, &QAbstractButton::pressed, this, [this]() {
+        makeActive(true);
+    });
+    connect(m_split->menu(), &QMenu::aboutToShow, this, [this]() {
+        makeActive(true);
+    });
+
     m_urlBar = new KateUrlBar(this);
 
     // like other editors, we try to re-use documents, of not modified
@@ -244,18 +255,14 @@ bool KateViewSpace::eventFilter(QObject *obj, QEvent *event)
         }
 
         // on mouse press on view space bar tool buttons: activate this space
-        if (button && !isActiveSpace() && event->type() == QEvent::MouseButtonPress) {
-            m_viewManager->setActiveSpace(this);
-            if (currentView()) {
-                m_viewManager->activateView(currentView()->document(), this);
-            }
+        if (event->type() == QEvent::MouseButtonPress) {
+            makeActive(true);
         }
     }
 
     // ensure proper view space activation even for widgets
-    if (event->type() == QEvent::FocusIn && !isActiveSpace() && currentWidget()) {
-        m_viewManager->setActiveSpace(this);
-        activateWidget(currentWidget());
+    if (event->type() == QEvent::FocusIn) {
+        makeActive(true);
         return false;
     }
 
@@ -362,10 +369,11 @@ KTextEditor::View *KateViewSpace::createView(KTextEditor::Document *doc)
     // restore the config of this view if possible
     if (!m_group.isEmpty()) {
         if (KateSession::Ptr as = KateApp::self()->sessionManager()->activeSession(); as->config()) {
-            // try id, fallback to url for old configs
+            // try id first
             QString id = QString::number(KateApp::self()->documentManager()->documentInfo(v->document())->sessionConfigId);
             QString vgroup = QStringLiteral("%1 %2").arg(m_group, id);
             if (!as->config()->hasGroup(vgroup)) {
+                // fallback to url, see KateViewSpace::saveViewConfig
                 vgroup = QStringLiteral("%1 %2").arg(m_group, v->document()->url().toString());
             }
             if (as->config()->hasGroup(vgroup)) {
@@ -562,8 +570,12 @@ void KateViewSpace::makeActive(bool focusCurrentView)
 {
     if (!isActiveSpace()) {
         m_viewManager->setActiveSpace(this);
-        if (focusCurrentView && currentView()) {
-            m_viewManager->activateView(currentView()->document(), this);
+        if (focusCurrentView) {
+            if (currentView()) {
+                m_viewManager->activateView(currentView()->document(), this);
+            } else if (currentWidget()) {
+                activateWidget(currentWidget());
+            }
         }
     }
     Q_ASSERT(isActiveSpace());
@@ -604,12 +616,9 @@ void KateViewSpace::registerDocument(KTextEditor::Document *doc)
      */
     connect(doc, &KTextEditor::Document::documentNameChanged, this, &KateViewSpace::updateDocumentName);
     connect(doc, &KTextEditor::Document::documentUrlChanged, this, &KateViewSpace::updateDocumentUrl);
-    connect(doc, &KTextEditor::Document::modifiedChanged, this, [this](KTextEditor::Document *doc) {
-        int tab = m_tabBar->documentIdx(doc);
-        if (tab >= 0) {
-            m_tabBar->setModifiedStateIcon(tab, doc);
-        }
-    });
+    connect(doc, &KTextEditor::Document::modifiedChanged, this, &KateViewSpace::updateDocumentIcon);
+    // needed to get mime-type udate right, see bug 489452
+    connect(doc, &KTextEditor::Document::reloaded, this, &KateViewSpace::updateDocumentIcon);
 
     /**
      * allow signals again, now that the tab is there
@@ -811,6 +820,15 @@ void KateViewSpace::updateDocumentName(KTextEditor::Document *doc)
     }
 }
 
+void KateViewSpace::updateDocumentIcon(KTextEditor::Document *doc)
+{
+    // update tab button if available, might not be the case for tab limit set!
+    const int buttonId = m_tabBar->documentIdx(doc);
+    if (buttonId >= 0) {
+        m_tabBar->setModifiedStateIcon(buttonId, doc);
+    }
+}
+
 void KateViewSpace::updateDocumentUrl(KTextEditor::Document *doc)
 {
     // update tab button if available, might not be the case for tab limit set!
@@ -985,6 +1003,7 @@ bool KateViewSpace::activateWidget(QWidget *widget)
     m_registeredDocuments.removeOne(widget);
     m_registeredDocuments.push_back(widget);
     m_tabBar->setCurrentDocument(DocOrWidget(widget));
+    widget->setFocus();
     return true;
 }
 
@@ -1005,6 +1024,17 @@ void KateViewSpace::setScrollSyncToolButtonVisible(bool visible)
     m_scrollSync->setHidden(!visible);
 }
 
+void KateViewSpace::detachDocument(KTextEditor::Document *doc)
+{
+    auto mainWindow = viewManager()->mainWindow()->newWindow();
+    mainWindow->viewManager()->openViewForDoc(doc);
+
+    // use single shot as this action can trigger deletion of this viewspace!
+    QTimer::singleShot(0, this, [this, doc]() {
+        closeDocument(doc);
+    });
+}
+
 void KateViewSpace::addPositionToHistory(const QUrl &url, KTextEditor::Cursor c, bool calledExternally)
 {
     // We don't care about invalid urls (Fixed Diff View / Untitled docs)
@@ -1118,6 +1148,7 @@ void KateViewSpace::showContextMenu(int idx, const QPoint &globalPos)
     QAction *aCloseTab = menu.addAction(QIcon::fromTheme(QStringLiteral("tab-close")), i18n("&Close Document"));
     QAction *aCloseOthers = menu.addAction(QIcon::fromTheme(QStringLiteral("tab-close-other")), i18n("Close Other &Documents"));
     QAction *aCloseAll = menu.addAction(QIcon::fromTheme(QStringLiteral("tab-close-all")), i18n("Close &All Documents"));
+    menu.addAction(m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("reopen_latest_closed_document")));
     menu.addSeparator();
     menu.addAction(KStandardAction::open(m_viewManager, &KateViewManager::slotDocumentOpen, this));
     QAction *aDetachTab = menu.addAction(QIcon::fromTheme(QStringLiteral("tab-detach")), i18n("D&etach Document"));
@@ -1212,43 +1243,55 @@ void KateViewSpace::showContextMenu(int idx, const QPoint &globalPos)
                                      QMessageBox::StandardButton::Ok);
         }
     } else if (choice == aDetachTab) {
-        auto mainWindow = viewManager()->mainWindow()->newWindow();
-        mainWindow->viewManager()->openViewForDoc(doc);
-
-        // use single shot as this action can trigger deletion of this viewspace!
-        QTimer::singleShot(0, this, [this, idx]() {
-            closeTabRequest(idx);
-        });
+        detachDocument(doc);
     }
 }
 
 void KateViewSpace::saveConfig(KConfigBase *config, int myIndex, const QString &viewConfGrp)
 {
     const QString groupname = QString(viewConfGrp + QStringLiteral("-ViewSpace %1")).arg(myIndex);
+    // Ensure that we store the group name. It is used to save config for views inside
+    // the viewspace. See KateViewSpace::saveViewConfig that is called when a view is closed
     m_group = groupname;
 
-    // aggregate all registered documents & views in view space (LRU ordered)
+    // aggregate all registered documents & views in view space
     // we need even the documents without tabs to avoid that we later have issues with closing
     // documents and not the right ones being used as replacement if you have a limit on tabs
+    // we first store all stuff without visible tabs and then the tabs in tab order to proper restore
+    // that order
     std::vector<KTextEditor::View *> views;
-    QStringList lruList;
-    const auto docList = m_tabBar->lruSortedDocuments();
-    for (auto doc : docList) {
+    QStringList docList;
+    const auto handleDoc = [this, &views, &docList](auto doc) {
         // we can only store stuff about documents that get an id
         const int sessionId = KateApp::self()->documentManager()->documentInfo(doc)->sessionConfigId;
         if (sessionId < 0) {
-            continue;
+            return;
         }
 
-        lruList << QString::number(sessionId);
+        docList << QString::number(sessionId);
         auto it = m_docToView.find(doc);
         if (it != m_docToView.end()) {
             views.push_back(it->second);
         }
+    };
+
+    const auto lruDocList = m_tabBar->lruSortedDocuments();
+    const auto tabs = m_tabBar->documentList();
+    for (auto doc : lruDocList) {
+        // skip stuff with visible tabs
+        if (std::find(tabs.begin(), tabs.end(), DocOrWidget(doc)) == tabs.end()) {
+            handleDoc(doc);
+        }
+    }
+    for (auto docOrWidget : tabs) {
+        // only docs are of interest
+        if (auto doc = docOrWidget.doc()) {
+            handleDoc(doc);
+        }
     }
 
     KConfigGroup group(config, groupname);
-    group.writeEntry("Documents", lruList);
+    group.writeEntry("Documents", docList);
     group.writeEntry("Count", static_cast<int>(views.size()));
 
     if (currentView()) {
@@ -1285,9 +1328,9 @@ void KateViewSpace::restoreConfig(KateViewManager *viewMan, const KConfigBase *c
     // set back bar status to configured variant
     tabBarToggled();
 
-    // restore Document lru list so that all tabs from the last session reappear
-    const QStringList lruList = group.readEntry("Documents", QStringList());
-    for (const auto &idOrUrl : lruList) {
+    // restore Document list so that all tabs from the last session reappear
+    const QStringList docList = group.readEntry("Documents", QStringList());
+    for (const auto &idOrUrl : docList) {
         // ignore stuff with no id or url
         if (idOrUrl.isEmpty()) {
             continue;
