@@ -230,6 +230,8 @@ public:
      */
     void setScrollSyncToolButtonVisible(bool visible);
 
+    void detachDocument(KTextEditor::Document *doc);
+
 protected:
     // DND
     void dragEnterEvent(QDragEnterEvent *e) override;
@@ -242,6 +244,7 @@ Q_SIGNALS:
 public:
     void documentDestroyed(QObject *doc);
     void updateDocumentName(KTextEditor::Document *doc);
+    void updateDocumentIcon(KTextEditor::Document *doc);
     void updateDocumentUrl(KTextEditor::Document *doc);
 
 private:
