@@ -19,7 +19,10 @@
 #include <KSharedConfig>
 
 #include <QCoreApplication>
+#include <QDir>
 #include <QEvent>
+#include <QFileInfo>
+#include <QMetaObject>
 #include <QPainter>
 #include <QPointer>
 #include <QSortFilterProxyModel>
@@ -266,8 +269,6 @@ private:
     FilterMode m_filterMode;
 };
 
-Q_DECLARE_METATYPE(QPointer<KTextEditor::Document>)
-
 KateQuickOpen::KateQuickOpen(KateMainWindow *mainWindow)
     : QFrame(mainWindow)
     , m_mainWindow(mainWindow)
@@ -431,7 +432,22 @@ void KateQuickOpen::slotReturnPressed()
     if (auto doc = index.data(KateQuickOpenModel::Document).value<KTextEditor::Document *>()) {
         view = m_mainWindow->activateView(doc);
     } else {
-        view = m_mainWindow->wrapper()->openUrl(index.data(Qt::UserRole).toUrl());
+        QFileInfo file(index.data(Qt::UserRole).toUrl().toLocalFile());
+        // If the file is a directory, we're likely opening a project
+        if (file.isDir()) {
+            // Make sure projectview exists, then invoke switchToProject method with the directory we have
+            QObject *projectView = m_mainWindow->pluginView(QStringLiteral("kateprojectplugin"));
+            if (projectView) {
+                QMetaObject::invokeMethod(projectView, "switchToProject", Qt::DirectConnection, QDir(index.data(Qt::UserRole).toUrl().toLocalFile()));
+                auto toolview = m_mainWindow->toolviewForName(QStringLiteral("kate_mdi_focus_toolview_projects"));
+                if (toolview) {
+                    m_mainWindow->showToolView(toolview);
+                    toolview->setFocus();
+                }
+            }
+        } else {
+            view = m_mainWindow->wrapper()->openUrl(index.data(Qt::UserRole).toUrl());
+        }
     }
 
     const auto strs = m_inputLine->text().split(QLatin1Char(':'));
@@ -473,6 +489,10 @@ void KateQuickOpen::slotReturnPressed()
 
 void KateQuickOpen::slotListModeChanged(KateQuickOpenModel::List mode)
 {
+    if (m_model->listMode() == mode) {
+        return;
+    }
+
     m_model->setListMode(mode);
     // this changes things again, needs refresh, let's go all the way
     updateState();
@@ -537,5 +557,3 @@ void KateQuickOpen::updateViewGeometry()
     const QPoint position{boundingRect.center().x() - size.width() / 2, boundingRect.y() + 6};
     move(position);
 }
-
-#include "moc_katequickopen.cpp"
