@@ -3,6 +3,7 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 #include "tooltip.h"
+#include "hintstate.h"
 
 #include <QApplication>
 #include <QEvent>
@@ -33,11 +34,27 @@ public:
 
     void highlightBlock(const QString &text) override
     {
-        const auto fmt = currentBlock().blockFormat();
+        QTextBlock block = currentBlock();
+        QTextBlockFormat fmt = block.blockFormat();
         if (fmt.property(QTextFormat::BlockCodeLanguage).isValid()) {
             // highlight blocks marked with BlockCodeLanguage format
             return KSyntaxHighlighting::SyntaxHighlighter::highlightBlock(text);
         }
+
+        const QList<QTextLayout::FormatRange> textFormats = block.textFormats();
+        for (const auto &f : textFormats) {
+            if (f.format.isAnchor()) {
+                QTextCharFormat charFmt = format(f.start);
+                charFmt.setFontUnderline(true);
+                // charFmt.setForeground(linkColor);
+                setFormat(f.start, f.length, charFmt);
+            } else if (f.format.fontFixedPitch()) {
+                QTextCharFormat charFmt = format(f.start);
+                charFmt.setBackground(m_inlineCodeSpanColor);
+                charFmt.setFont(m_editorFont);
+                setFormat(f.start, f.length, charFmt);
+            }
+        }
     }
 
     void applyFormat(int offset, int length, const KSyntaxHighlighting::Format &format) override
@@ -47,57 +64,14 @@ public:
         }
         KSyntaxHighlighting::SyntaxHighlighter::applyFormat(offset, length, format);
     }
+
+    QBrush m_inlineCodeSpanColor;
+    QFont m_editorFont;
 };
 
 class TooltipPrivate : public QTextBrowser
 {
 public:
-    void setTooltipText(const QString &text, TextHintMarkupKind kind)
-    {
-        if (text.isEmpty())
-            return;
-
-        m_kind = kind;
-        // we have to do this to handle soft line
-        if (kind == TextHintMarkupKind::PlainText) {
-            setPlainText(text);
-        } else {
-            QString htext = text;
-            setMarkdown(htext);
-        }
-        resizeTip(text);
-    }
-
-    void appendMarkdown(const QString &text)
-    {
-        auto md = toMarkdown();
-        // hbreak
-        md += QStringLiteral("\n----\n");
-        md += text;
-        setMarkdown(md);
-    }
-
-    void appendTooltipText(const QString &text, TextHintMarkupKind kind)
-    {
-        auto cursor = textCursor();
-        cursor.movePosition(QTextCursor::End);
-        if (m_kind == TextHintMarkupKind::PlainText && kind == TextHintMarkupKind::PlainText) {
-            cursor.insertText(QStringLiteral("\n"));
-            cursor.insertText(text);
-        } else if (m_kind == TextHintMarkupKind::MarkDown && kind == TextHintMarkupKind::MarkDown) {
-            appendMarkdown(text);
-        } else if (m_kind == TextHintMarkupKind::PlainText && kind == TextHintMarkupKind::MarkDown) {
-            appendMarkdown(text);
-        } else if (m_kind == TextHintMarkupKind::MarkDown && kind == TextHintMarkupKind::PlainText) {
-            appendMarkdown(text);
-        }
-
-        // resize if too small
-        if (height() < 300) {
-            resize(width(), height() + 200);
-        }
-    }
-
     void setView(KTextEditor::View *view)
     {
         // view changed?
@@ -110,7 +84,7 @@ public:
 
             m_view = view;
 
-            hl->setDefinition(KTextEditor::Editor::instance()->repository().definitionForFileName(m_view->document()->url().toString()));
+            m_hl->setDefinition(KTextEditor::Editor::instance()->repository().definitionForFileName(m_view->document()->url().toString()));
 
             if (m_view && m_view->focusProxy()) {
                 m_view->focusProxy()->installEventFilter(this);
@@ -118,16 +92,18 @@ public:
         }
     }
 
-    TooltipPrivate(QWidget *parent, bool manual)
+    TooltipPrivate(QWidget *parent, bool manual, KTextEditor::Range wordRange)
         : QTextBrowser(parent)
-        , hl(new TooltipHighlighter(document()))
+        , m_hl(new TooltipHighlighter(document()))
         , m_manual(manual)
+        , m_hoveredWordRange(wordRange)
     {
         setWindowFlags(Qt::FramelessWindowHint | Qt::BypassGraphicsProxyWidget | Qt::ToolTip);
         setAttribute(Qt::WA_DeleteOnClose, true);
         document()->setDocumentMargin(5);
         setFrameStyle(QFrame::Box | QFrame::Raised);
         connect(&m_hideTimer, &QTimer::timeout, this, &TooltipPrivate::hideTooltip);
+        setWordWrapMode(QTextOption::WordWrap);
 
         setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
         setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
@@ -137,16 +113,32 @@ public:
 
         auto updateColors = [this](KTextEditor::Editor *e) {
             auto theme = e->theme();
-            hl->setTheme(theme);
+            m_hl->setTheme(theme);
 
             auto pal = palette();
             const QColor bg = theme.editorColor(KSyntaxHighlighting::Theme::BackgroundColor);
-            pal.setColor(QPalette::Base, bg);
             const QColor normal = theme.textColor(KSyntaxHighlighting::Theme::Normal);
+            const QColor separator = bg.lightness() < 127 ? normal.darker() : normal.lighter(180);
+            // Frame color
+            pal.setColor(QPalette::Dark, separator);
+            pal.setColor(QPalette::Light, separator);
+
+            pal.setColor(QPalette::Base, bg);
             pal.setColor(QPalette::Text, normal);
             setPalette(pal);
 
-            setFont(KTextEditor::Editor::instance()->font());
+            if (bg.lightness() < 127) {
+                m_hl->m_inlineCodeSpanColor = bg.lighter();
+            } else {
+                m_hl->m_inlineCodeSpanColor = bg.darker(120);
+            }
+
+            auto font = KTextEditor::Editor::instance()->font();
+            if (font.pointSize() > 11) {
+                font.setPointSize(font.pointSize() - 1);
+            }
+            setFont(font);
+            m_hl->m_editorFont = font;
         };
         updateColors(KTextEditor::Editor::instance());
         connect(KTextEditor::Editor::instance(), &KTextEditor::Editor::configChanged, this, updateColors);
@@ -172,9 +164,17 @@ public:
             if (o == verticalScrollBar() || o == horizontalScrollBar()) {
                 return false;
             }
+
+            auto pt = static_cast<QSinglePointEvent *>(e)->globalPosition().toPoint();
+            auto cursor = m_view->coordinatesToCursor(m_view->mapFromGlobal(pt));
+            // we are hovering over the word for which this tooltip exists, dont hide
+            if (m_hoveredWordRange == m_view->document()->wordRangeAt(cursor)) {
+                return false;
+            }
+
             auto pos = mapFromGlobal(static_cast<QSinglePointEvent *>(e)->globalPosition()).toPoint();
             if (!m_manual && !hasFocus() && !rect().contains(pos)) {
-                hideTooltipWithDelay();
+                hideTooltip();
             }
         } break;
         case QEvent::MouseButtonPress:
@@ -202,24 +202,26 @@ public:
         m_hideTimer.start(100);
     }
 
-    void resizeTip(const QString &text)
+    void resizeTip()
     {
-        QFontMetrics fm(font());
-        QSize size = fm.size(0, text);
-
-        const int height = fm.lineSpacing() * (document()->lineCount());
-
-        size.setHeight(std::min(height, m_view->height() / 3));
-        size.setWidth(std::min<int>(size.width(), m_view->width() / 2.5));
+        // get a natural size for the document
+        document()->adjustSize();
+        QSize size = document()->size().toSize();
 
         const int contentsMarginsWidth = this->contentsMargins().left() + this->contentsMargins().right();
         const int contentsMarginsHeight = this->contentsMargins().top() + this->contentsMargins().top();
         const int docMargin = 2 * document()->documentMargin();
         const int wMargins = contentsMarginsWidth + docMargin + verticalScrollBar()->height();
-        const int hMargins = contentsMarginsHeight + docMargin + horizontalScrollBar()->height();
+        const int hMargins = contentsMarginsHeight + docMargin + horizontalScrollBar()->width();
 
-        size.setWidth(size.width() + wMargins);
+        // add internal document padding and possible scrollbars to size
         size.setHeight(size.height() + hMargins);
+        size.setWidth(size.width() + wMargins);
+
+        // limit the tool tip size; the resize call will respect these limits
+
+        const auto fullWith = m_view->window() ? m_view->window()->width() : m_view->width();
+        setMaximumSize(fullWith / 2.5, m_view->height() / 3);
         resize(size);
     }
 
@@ -258,6 +260,18 @@ public:
         this->move(p);
     }
 
+    void upsert(size_t instanceId, const QString &text, TextHintMarkupKind kind)
+    {
+        m_hintState.upsert(instanceId, text, kind);
+        triggerChange();
+    }
+
+    void remove(size_t instanceId)
+    {
+        m_hintState.remove(instanceId);
+        triggerChange();
+    }
+
 protected:
     void enterEvent(QEnterEvent *event) override
     {
@@ -296,33 +310,88 @@ protected:
     }
 
 private:
+    void triggerChange()
+    {
+        m_hintState.render([this](const HintState::Hint &data) {
+            const auto &[text, kind] = data;
+            if (kind == TextHintMarkupKind::PlainText) {
+                setPlainText(text);
+            } else {
+                setMarkdown(text);
+            }
+
+            auto block = document()->firstBlock();
+            for (int i = 0; i < document()->blockCount(); ++i) {
+                auto bfmt = block.blockFormat();
+                // Fix some things for code blocks in markdown
+                if (bfmt.hasProperty(QTextFormat::BlockCodeLanguage)) {
+                    QTextCursor c(block);
+                    // allow word wrap
+                    bfmt.setNonBreakableLines(false);
+                    // fix the font, use our own mono font
+                    c.select(QTextCursor::BlockUnderCursor);
+                    auto cfmt = block.charFormat();
+                    cfmt.setFont(this->font());
+                    c.setBlockFormat(bfmt);
+                    c.setCharFormat(cfmt);
+                } else if (bfmt.headingLevel() != 0) {
+                    // Make all headings H3
+                    QTextCursor c(block);
+                    bfmt.setHeadingLevel(3);
+                    c.select(QTextCursor::BlockUnderCursor);
+                    QTextCharFormat cfmt = block.charFormat();
+                    cfmt.setProperty(QTextFormat::FontSizeAdjustment, 1);
+                    c.setBlockFormat(bfmt);
+                    c.setCharFormat(cfmt);
+                }
+                block = block.next();
+            }
+
+            resizeTip();
+        });
+    }
+
     bool inContextMenu = false;
     QPointer<KTextEditor::View> m_view;
     QTimer m_hideTimer;
-    KSyntaxHighlighting::SyntaxHighlighter *hl;
+    TooltipHighlighter *m_hl;
     bool m_manual;
-    TextHintMarkupKind m_kind = TextHintMarkupKind::PlainText;
+    HintState m_hintState;
+    const KTextEditor::Range m_hoveredWordRange;
 };
 
-void KateTooltip::show(const QString &text, TextHintMarkupKind kind, QPoint pos, KTextEditor::View *v, bool manual)
+QObject *KateTooltip::show(size_t instanceId,
+                           const QString &text,
+                           TextHintMarkupKind kind,
+                           QPoint pos,
+                           KTextEditor::View *v,
+                           bool manual,
+                           KTextEditor::Range hoveredRange)
 {
-    if (text.isEmpty())
-        return;
-
     if (!v || !v->document()) {
-        return;
+        return nullptr;
     }
 
     static QPointer<TooltipPrivate> tooltip = nullptr;
     if (tooltip && tooltip->isVisible()) {
-        tooltip->appendTooltipText(text, kind);
-        return;
+        if (text.isEmpty() || text.trimmed().isEmpty()) {
+            tooltip->remove(instanceId);
+            return tooltip;
+        }
+
+        tooltip->upsert(instanceId, text, kind);
+        return tooltip;
     }
     delete tooltip;
 
-    tooltip = new TooltipPrivate(v, manual);
+    if (text.isEmpty() || text.trimmed().isEmpty()) {
+        return tooltip;
+    }
+
+    tooltip = new TooltipPrivate(v, manual, hoveredRange);
     tooltip->setView(v);
-    tooltip->setTooltipText(text, kind);
+    tooltip->upsert(instanceId, text, kind);
     tooltip->place(pos);
     tooltip->show();
+    return tooltip;
 }
